# Create flat PDF by converting each input PDF page to a PNG
import os
from tempfile import mkdtemp
from pdf.utils.path import add_suffix
from pdf.conduit.upscale import upscale
from pdf.conduit.convert.img2pdf import IMG2PDF
from pdf.conduit.convert.pdf2img import PDF2IMG


class Flatten:
    def __init__(self, file_name, scale=2.0, suffix='flat', tempdir=None, progress_bar=None):
        """Create a flat single-layer PDF by converting each page to a PNG image"""
        self._file_name = file_name
        self.tempdir = tempdir if tempdir else mkdtemp()
        self.suffix = suffix
        self.directory = os.path.dirname(file_name)
        self.progress_bar = progress_bar

        if scale and scale is not 0 and scale is not 1.0:
            self.file_name = upscale(file_name, scale=scale, tempdir=tempdir)
        else:
            self.file_name = self._file_name

        self.imgs = None
        self.pdf = None

    def __str__(self):
        return str(self.pdf)

    def get_imgs(self):
        self.imgs = PDF2IMG(self.file_name, tempdir=self.tempdir, progress_bar=self.progress_bar).save()
        return self.imgs

    def save(self, remove_temps=True):
        if self.imgs is None:
            self.get_imgs()
        i2p = IMG2PDF(self.imgs, self.directory, self.tempdir, self.progress_bar)
        self.pdf = i2p.save(remove_temps=remove_temps, output_name=add_suffix(self._file_name, self.suffix))
        return self.pdf



# Convert a PNG image file to a PDF
import os
import shutil
import PySimpleGUI as gui
from PIL import Image
from tqdm import tqdm
from pdf.conduit.watermark.canvas import CanvasImg, CanvasObjects
from pdf.conduit.watermark.draw import WatermarkDraw
from pdf.conduit.merge import Merge


def clean_temps(tempdir):
    # TODO: Files remaining open on window and preventing deletion
    if os.path.isdir(tempdir):
        try:
            shutil.rmtree(tempdir)
        except PermissionError:
            print(tempdir, 'was not removed')


class IMG2PDF:
    def __init__(self, imgs, destination=None, tempdir=None, progress_bar=None):
        """Convert each image into a PDF page and merge all pages to one PDF file"""
        self.imgs = imgs
        self.output_dir = destination
        self.tempdir = tempdir
        self.progress_bar = progress_bar

        self.pdf_pages = self.img2pdf()

    def img2pdf(self):
        # TODO: Figure out weather this method is causing unclosed file warnings and errors
        # PySimpleGUI progress bar
        if self.progress_bar is 'gui':
            pdfs = []
            for index, i in enumerate(self.imgs):
                with Image.open(i) as im:
                    width, height = im.size

                    co = CanvasObjects()
                    co.add(CanvasImg(i, 1.0, w=width, h=height))

                    pdf = WatermarkDraw(co, tempdir=self.tempdir, pagesize=(width, height)).write()
                    pdfs.append(pdf)
                if not gui.OneLineProgressMeter('Saving PNGs as flat PDFs', index + 1, len(self.imgs),
                                                orientation='h', key='progress'):
                    break
            return pdfs

        # TQDM progress bar
        elif self.progress_bar is 'tqdm':
            loop = tqdm(self.imgs, desc='Saving PNGs as flat PDFs', total=len(self.imgs), unit='PDFs')

        # No progress bar
        else:
            loop = self.imgs
        pdfs = []
        for i in loop:
            with Image.open(i) as im:
                width, height = im.size

            co = CanvasObjects()
            co.add(CanvasImg(i, 1.0, w=width, h=height))

            pdf = WatermarkDraw(co, tempdir=self.tempdir, pagesize=(width, height)).write()
            pdfs.append(pdf)
        return pdfs

    def save(self, remove_temps=True, output_name='merged imgs'):
        m = str(Merge(self.pdf_pages, output_name=output_name, output_dir=self.output_dir))
        if remove_temps:
            clean_temps(self.tempdir)
        return m


def img2pdf(imgs, output_name='merged_imgs', destination=None, tempdir=None, progress_bar=None):
    return IMG2PDF(imgs, destination, tempdir, progress_bar).save(output_name)



from pdf.utils import *
from .encrypt import Encrypt
from .merge import Merge
from .slice import slicer
from .flatten import Flatten
from .upscale import upscale
from .rotate import rotate
from .watermark import WatermarkAdd, Watermark, Label
from .convert import pdf2img, img2pdf


__all__ = ["upscale", "rotate", "Encrypt", "Merge", "Watermark", "Label", "WatermarkAdd", "slicer", "Info", "Flatten",
           "pdf2img", "img2pdf"]
